#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os;
import dircache;
import sys;
import urllib
import traceback
import glob

html_copyright = """<div style="font-size: small; color: gray; margin-top: 1em;">Authors: Michal Sojka, Pavel Píša, Copyright © 2010, 2011 Czech Technical University in Prague</div>"""

class DimValue(object):
    def __new__(cls, dim, value):
        if value in dim:
            return dim[value]
        else:
            return super(DimValue, cls).__new__(cls)
    def __init__(self, dim, value):
	self.dim = dim
	self.value = value
        self.dim.addValue(self)
    def __str__(self):
        return self.dim.val2str(self.value)
    def __repr__(self):
	return "DimValue(%s, %s)" % (repr(self.dim), repr(self.value))
    def htmlTableHeading(self):
	return self.dim.htmlTableHeading(self.value)
    def index(self):
        return self.dim.sortedKeys.index(self.value)

class DimValues(list):
    def replace(self, val):
        for i in xrange(len(self)):
            if self[i].dim == val.dim:
                self[i] = val
    def __add__(self, val):
        ret = DimValues(self)
        ret.append(val)
        return ret
    def __sub__(self, dim):
        result = DimValues(self)
        for v in self:
            if v.dim == dim:
                result.remove(v)
        return result
    def key(self):
        return tuple([v.value for v in self])

class Dimension(dict):
    def __init__(self, atype, name=None):
	self.type = atype
	if (name):
	    self.name = name
	else:
	    self.name = atype
        self.sortedKeys = []

    def __iter__(self):
	for i in xrange(len(self)):
	    yield self.getValue(i)
    def getValue(self, index):
        return self[self.sortedKeys[index]]

    def addValue(self, value):
        if value not in self:
            if isinstance(value, DimValue):
                self[value.value] = value
            else:
                raise Exception("Unsupported usage of addValue")
                #self[value] = DimValue(self, value)
            self.sortedKeys = self.keys()
            self.sortedKeys.sort()
    def val2str(self, v):
        return str(v)
    def htmlTableHeading(self, v):
	return self.val2str(v)
    def __str__(self):
        return self.name
    def __repr__(self):
        return "Dimension(%s)"%self.type

class DimensionKern(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'gwkern', 'GW kernel')
    def htmlTableHeading(self, v):
	i=v.find(":")
	if i>0: kver=v[:i]
	else: kver=v
	return v+"<br><a href='config-%s'>config</a>"%(urllib.quote(kver))
    def versions(self):
	for v in self.values:
	    i=v.find(":")
	    if i>0: kver=v[:i]
	    else: kver=v
	    yield kver

class DimensionHostKern(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'hostkern', 'Host kernel')
    def val2str(self, v):
        if v.find("host-") == 0:
            v = v[5:]
        return v
    def htmlTableHeading(self, v):
        v = self.val2str(v)
	i = v.find(":")
	if i>0: kver = v[:i]
        else: kver = v
	return v+"<br><a href='config-%s'>config</a>"%(urllib.quote(kver))
    def versions(self):
	for v in self.values:
	    i=v.find(":")
	    if i>0: kver=v[:i]
	    else: kver=v
	    yield kver

class DimensionTest(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'test', 'Test')
    def htmlTableHeading(self, v):
	return v+"<br><a href='%s.sh.html'>source</a>"%(urllib.quote(v))

class DimensionLoad(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'load', 'Load')

class DimensionTraffic(Dimension):
    def __init__(self):
	Dimension.__init__(self, 'traf', 'Traffic')
    def val2str(self, v):
        if v == "50":
            return "50%"
        elif v == "oneatatime":
            return "one message at a time"
        else:
            return v
    def htmlTableHeading(self, v):
        return self.val2str(v)
class Test(object):
    @classmethod
    def isOnPath(cls, path):
	f = os.path.join(path, 'plot.sh')
	return os.path.isfile(f)
    def __init__(self, path, values, tests=None):
	self.path = path
        self.name = os.path.basename(path)
        self.values = values
        self.tests = tests
    def printThumbLink(self, file):
#         try:
#             imgs = [img for img in dircache.listdir(thumb)]
#         except OSError:
#             imgs = [ self.name + ".png" ]
        imgs = [ 'tgraph.png' ]
        for img in imgs:
            print >>file, "<a href='%s/results.html'><img src='%s/%s'></a>" % \
                  (urllib.quote(self.path), urllib.quote(self.path), img)
    def fullImgLink(self, pngName):
        if 'NO_PDF' in os.environ:
            return "<div><img src='%s' /></div>" % pngName
        else:
            return "<div><a href='%s'><img src='%s' /></a></div>" % \
                   (pngName[:-4]+".pdf", pngName)
        
    def htmlPrintStats(self, html):
        cwd = os.getcwd()
        os.chdir(self.path)
        stats = glob.glob("*-stat.txt")
        print >>html, "<h3>Statistics</h3>"
        print >>html, "<table><tr>"
        stats.sort()
        for i in stats:
            lines = open(i).readlines()
            def fixupLine(l):
                comment = l.find("#")
                if comment >= 0:
                    l = l[:comment-1]
                if l.find("cmdline=") == 0:
                    l = "<abbr title=%s>cmdline=...</abbr>" % str(l[8:])
                return l
            lines = [fixupLine(l) for l in lines]
            print >>html, "<td><h4>%s</h4>%s</td>" % (i, "<br />".join(lines))
        print >>html, "</tr></table>"
        os.chdir(cwd)
        
    def generateHtml(self):
        html = open(os.path.join(self.path, 'results.html'), "w")
	title = "CAN gateway timing analysis"
        cdup = "../"*len(self.values)
	print >> html, """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<title>%s</title>
<link rel="stylesheet" href="%sstyle.css" type="text/css" /> 
</head>
<body>
<h1>%s</h1>"""  % (title, cdup, title)
        params = ["%s %s" % (v.dim, v) for v in self.values]
        print >>html, "Results for:", ", ".join(params)
        print >>html, "<div class='otherview'><h4>Other results</h4><table>"
	for d in self.tests.space:
            links = []
            for v in d:
                if v in self.values:
                    links.append("<span class='value current'>%s</span>"%str(v))
                else:
                    vv = DimValues(self.values)
                    vv.replace(v)
                    try:
                        href = cdup + urllib.quote(self.tests[vv.key()].path+"/results.html")
                        links.append("<span class='value other'><a href='%s'>%s</a></span>"%(href, str(v)))
                    except KeyError:
                        links.append("<span class='value missing'>%s</span>"%str(v))
            print >>html, "<tr><th>%s</th><td>" % d, " ".join(links), "</td></tr>"

        print >>html, "</table></div>"
        print >>html, self.fullImgLink("graph.png")
        self.htmlPrintStats(html)
        cwd = os.getcwd()
        os.chdir(self.path)
        additionalImgs = glob.glob("graph?*.png")
        if additionalImgs: print >>html, "<h3>Additional graphs</h3>"
        for i in additionalImgs:
            print >>html, "<h4>%s</h4>" % i[5:-4]
            print >>html, self.fullImgLink(i)
        os.chdir(cwd)
        
        print >>html, "<hr />"
        print >>html, "<a href='./'>Raw data</a><br />"
        print >>html, "<a href='%s'>Script source</a><br />" % (cdup+self.name+".sh.html")
        print >>html, "<a href='%s'>Back to top</a><br />" % cdup
        print >>html,"%s</body></html>" % html_copyright
        html.close()

class Space(list):
    """List of Dimensions()s (order matters)"""
    def __init__(self, *dimensions):
        self.extend(list(dimensions))
    def path2dimValues(self, path):
        coordinates = path.split("/")
        if len(coordinates) != len(self):
            raise KeyError("The number coordinates do not match the number of dimensions: " + str(coordinates))
        
        dv = DimValues([DimValue(self[i], coordinates[i]) \
                            for i in xrange(len(coordinates))])
        return dv

    def iterValues(self):
        idx = [0 for i in xrange(len(self))]
        done=False
        while not done:
            values=DimValues()
            for i in xrange(len(self)):
                values.append(self[i].values()[idx[i]])
            yield values
            done=True
            for i in xrange(len(self)):
                idx[i] += 1
                if idx[i] < len(self[i]):
                    done=False
                    break
                idx[i] = 0
    def reorder(self, dimValues):
        reordered = DimValues()
        for d in self:
	    for v in dimValues:
		if v.dim == d:
		    reordered.append(v)
        return reordered
    def iterDimensionPairs(self):
	for i in xrange(len(self)):
	    for j in xrange(i+1, len(self)):
		yield (self[i], self[j])
		yield (self[j], self[i])
    def iterRemainingDimensions(self, dimensionPair):
	for d in self:
	    if d not in dimensionPair:
		yield d


class Tests(dict):
    """Represents all tests organized along several dimensions"""
    def __init__(self, rootpath, space):
	dict.__init__(self)
	self.space = space
	if (rootpath):
	    self.populate(rootpath)
    def getTest(self, key):
	if len(key) != len(self.space):
	    raise KeyError("The coordinates in key do not match the dimension of the space")
        realkey = self.space.reorder(key)
	return self[realkey.key()]

    def addTest(self, test):
	self[test.values.key()] = test
	
    def populate(self, rootpath):
	for root, dirs, files in os.walk(rootpath):
	    if (root.find(rootpath) == 0):
		path = root[len(rootpath):]
	    else:
		path = rootpath
	    if Test.isOnPath(root):
                dv = self.space.path2dimValues(path)
		self.addTest(Test(root, dv, self))
    def generateHtml(self):
	for pair in self.space.iterDimensionPairs():
	    remDims = Space(*tuple([d for d in self.space.iterRemainingDimensions(pair)]))
	    for vals in remDims.iterValues():
		page = Page(pair, vals, self)
                print page.getName()
		page.generate()
	try:
	    os.remove("index.html")
	except OSError: pass
	os.symlink(page.getName(), "index.html")
        css = open("style.css", "w")
        print >>css, """img { border: 0; }
table { border-collapse: collapse; }
th, td { border: 1px solid lightgray; padding: 4px;}
h4 { margin: 0; }
.otherview { margin: 1ex 0}
.otherview .value { color: black; padding: 0ex 1ex; -moz-border-radius: 1ex; border-radius: 1ex;}
.otherview .value a { color: inherit; text-decoration: none; }
.otherview .other:hover { background: #eee; }
.otherview .missing { color: gray; }
.otherview .current { background: #ccc; }
.na { width: 150px; height: 109px; display:  table-cell; text-align: center; vertical-align: middle; }
"""
        css.close()
        for test in self.values():
            print test.path
            test.generateHtml()

	os.system("source-highlight -d --output-dir=. ../*.sh > /dev/null")

class Page(object):
    def __init__(self, dimPair, valsOther, tests):
	self.dimy, self.dimx = dimPair
	self.dimOther = [v.dim for v in valsOther]
	self.valsOther = tests.space.reorder(valsOther)
	self.tests = tests
    def getName(self):
	return "%s-vs-%s-for-%s.html"%(self.dimy.type, self.dimx.type,
                                       "-".join(["%02d"%v.index() for v in self.valsOther]))
    def generate(self):
	html = open(self.getName(), "w")
	title = "CAN gateway timing analysis" 
	print >> html, """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<title>%s</title>
<link rel="stylesheet" href="style.css" type="text/css" /> 
</head>
<body>
<h1>%s</h1>"""  % (title, title)
        params = ["%s %s" % (v.dim, v) for v in self.valsOther]
        print >>html, "<h3>Results for ", ", ".join(params), "</h3>"
        print >>html, "<div class='otherview'><h4>Other views</h4>"
        print >>html, "<table><tr>"
	for d in self.dimOther:
            print >>html, "<th>%s</th>" % d
            print >>html, "<td><a href='%s'>&rarr;</a> " % \
                Page((self.dimy, d), self.valsOther - d + self.dimx.getValue(0), self.tests).getName()
            print >>html, "<a href='%s'>&darr;</a></td>" % \
                Page((d, self.dimx), self.valsOther - d + self.dimy.getValue(0), self.tests).getName()
            links = []
            print >>html, "<td>"
            for v in d:
                if v in self.valsOther:
                    links.append("<span class='value current'>%s</span>"%str(v))
                else:
                    vv = DimValues(self.valsOther)
                    vv.replace(v)
                    links.append("<span class='value other'><a href='%s'>%s</a></span>"%(urllib.quote(Page((self.dimy, self.dimx), vv, self.tests).getName()), str(v)))
            print >>html, " ".join(links)
            print >>html, "</td></tr>"
        print >>html, "</table></div>"

	print >>html, "<table><thead><tr><td>%s &rarr; <br />%s &darr;</td>" % (self.dimx.name, self.dimy.name)
	for x in self.dimx:
	    print >>html, "<th>%s</th>" % x.htmlTableHeading()
	print >>html, "</tr></thead>"
	for y in self.dimy:
	    print >>html, "<tr><th>%s</th>" % y.htmlTableHeading()

	    for x in self.dimx:
		print >>html, "<td>"
		idx = [x,y]
		idx.extend(self.valsOther)
                try:
                    test = tests.getTest(idx)
                    test.printThumbLink(html)
                except KeyError:
                    print >>html, "<span class='na'>N/A</span>"
                print >>html, "</td>"
	    print >>html, "</tr>"
	print >> html, """
</table>
%s
</body>
""" % html_copyright


if __name__ == "__main__":
    os.chdir(sys.argv[1])
    os.system("rm *.html")
    tests = Tests("./", Space(DimensionHostKern(), DimensionKern(), DimensionTraffic(), DimensionLoad(), DimensionTest()))
    tests.generateHtml()
    sys.exit(0)
